# Generated by Django 5.0.8 on 2024-09-18 08:57

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbaccounting", "0011_transaction"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name="entryaccountinginformation",
            name="external_invoice_users",
            field=models.ManyToManyField(
                blank=True,
                help_text="External users who are able to see the invoices generated for this counterparty",
                related_name="external_accounting_information",
                to=settings.AUTH_USER_MODEL,
                verbose_name="External User",
            ),
        ),
    ]
