# Generated by Django 5.0.3 on 2024-04-29 11:33

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbaccounting", "0009_invoicetype_alter_bookingentry_options_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="bookingentry",
            options={
                "permissions": (("can_generate_booking_entries", "Can Generate Bookings"),),
                "verbose_name": "Booking",
                "verbose_name_plural": "Bookings",
            },
        ),
    ]
