# Generated by Django 4.2.9 on 2024-02-20 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbaccounting", "0007_alter_invoice_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="invoice",
            options={
                "permissions": (("administrate_invoice", "Can Administrate Invoice"),),
                "verbose_name": "Invoice",
                "verbose_name_plural": "Invoices",
            },
        ),
    ]
